@@ -23,7 +23,14 @@
 """Contains functions for registering clients."""
 
 import logging
-from typing import TYPE_CHECKING, Iterable, List, Optional, Tuple, TypedDict
+from typing import (
+    TYPE_CHECKING,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    TypedDict,
+)
 
 from prometheus_client import Counter
 
@@ -44,12 +51,11 @@ from synapse.api.errors import (
 )
 from synapse.appservice import ApplicationService
 from synapse.config.server import is_threepid_reserved
-from synapse.handlers.device import DeviceHandler
 from synapse.http.servlet import assert_params_in_dict
+from synapse.metrics import SERVER_NAME_LABEL
 from synapse.replication.http.login import RegisterDeviceReplicationServlet
 from synapse.replication.http.register import (
     ReplicationPostRegisterActionsServlet,
-    ReplicationRegisterServlet,
 )
 from synapse.spam_checker_api import RegistrationBehaviour
 from synapse.types import GUEST_USER_ID_PATTERN, RoomAlias, UserID, create_requester
@@ -64,29 +70,38 @@ logger = logging.getLogger(__name__)
 registration_counter = Counter(
     "synapse_user_registrations_total",
     "Number of new users registered (since restart)",
-    ["guest", "shadow_banned", "auth_provider"],
+    labelnames=["guest", "shadow_banned", "auth_provider", SERVER_NAME_LABEL],
 )
 
 login_counter = Counter(
     "synapse_user_logins_total",
     "Number of user logins (since restart)",
-    ["guest", "auth_provider"],
+    labelnames=["guest", "auth_provider", SERVER_NAME_LABEL],
 )
 
 
-def init_counters_for_auth_provider(auth_provider_id: str) -> None:
+def init_counters_for_auth_provider(auth_provider_id: str, server_name: str) -> None:
     """Ensure the prometheus counters for the given auth provider are initialised
 
     This fixes a problem where the counters are not reported for a given auth provider
     until the user first logs in/registers.
+
+    Args:
+        auth_provider_id: The ID of the auth provider to initialise counters for.
+        server_name: Our server name (used to label metrics) (this should be `hs.hostname`).
     """
     for is_guest in (True, False):
-        login_counter.labels(guest=is_guest, auth_provider=auth_provider_id)
+        login_counter.labels(
+            guest=is_guest,
+            auth_provider=auth_provider_id,
+            **{SERVER_NAME_LABEL: server_name},
+        )
         for shadow_banned in (True, False):
             registration_counter.labels(
                 guest=is_guest,
                 shadow_banned=shadow_banned,
                 auth_provider=auth_provider_id,
+                **{SERVER_NAME_LABEL: server_name},
             )
 
 
@@ -99,6 +114,7 @@ class LoginDict(TypedDict):
 
 class RegistrationHandler:
     def __init__(self, hs: "HomeServer"):
+        self.server_name = hs.hostname
         self.store = hs.get_datastores().main
         self._storage_controllers = hs.get_storage_controllers()
         self.clock = hs.get_clock()
@@ -114,13 +130,11 @@ class RegistrationHandler:
         self._account_validity_handler = hs.get_account_validity_handler()
         self._user_consent_version = self.hs.config.consent.user_consent_version
         self._server_notices_mxid = hs.config.servernotices.server_notices_mxid
-        self._server_name = hs.hostname
         self._user_types_config = hs.config.user_types
 
         self._spam_checker_module_callbacks = hs.get_module_api_callbacks().spam_checker
 
         if hs.config.worker.worker_app:
-            self._register_client = ReplicationRegisterServlet.make_client(hs)
             self._register_device_client = RegisterDeviceReplicationServlet.make_client(
                 hs
             )
@@ -141,7 +155,9 @@ class RegistrationHandler:
         )
         self.refresh_token_lifetime = hs.config.registration.refresh_token_lifetime
 
-        init_counters_for_auth_provider("")
+        init_counters_for_auth_provider(
+            auth_provider_id="", server_name=self.server_name
+        )
 
     async def check_username(
         self,
@@ -365,6 +381,7 @@ class RegistrationHandler:
             guest=make_guest,
             shadow_banned=shadow_banned,
             auth_provider=(auth_provider_id or ""),
+            **{SERVER_NAME_LABEL: self.server_name},
         ).inc()
 
         # If the user does not need to consent at registration, auto-join any
@@ -425,7 +442,7 @@ class RegistrationHandler:
         if self.hs.config.registration.auto_join_user_id:
             fake_requester = create_requester(
                 self.hs.config.registration.auto_join_user_id,
-                authenticated_entity=self._server_name,
+                authenticated_entity=self.server_name,
             )
 
             # If the room requires an invite, add the user to the list of invites.
@@ -438,7 +455,7 @@ class RegistrationHandler:
             requires_join = True
         else:
             fake_requester = create_requester(
-                user_id, authenticated_entity=self._server_name
+                user_id, authenticated_entity=self.server_name
             )
 
         # Choose whether to federate the new room.
@@ -470,7 +487,7 @@ class RegistrationHandler:
 
                     await room_member_handler.update_membership(
                         requester=create_requester(
-                            user_id, authenticated_entity=self._server_name
+                            user_id, authenticated_entity=self.server_name
                         ),
                         target=UserID.from_string(user_id),
                         room_id=room_id,
@@ -496,7 +513,7 @@ class RegistrationHandler:
                     if requires_join:
                         await room_member_handler.update_membership(
                             requester=create_requester(
-                                user_id, authenticated_entity=self._server_name
+                                user_id, authenticated_entity=self.server_name
                             ),
                             target=UserID.from_string(user_id),
                             room_id=room_id,
@@ -542,7 +559,7 @@ class RegistrationHandler:
                 # we don't have a local user in the room to craft up an invite with.
                 requires_invite = await self.store.is_host_joined(
                     room_id,
-                    self._server_name,
+                    self.server_name,
                 )
 
                 if requires_invite:
@@ -559,7 +576,7 @@ class RegistrationHandler:
                         if join_rules_event:
                             join_rule = join_rules_event.content.get("join_rule", None)
                             requires_invite = (
-                                join_rule and join_rule != JoinRules.PUBLIC
+                                join_rule is not None and join_rule != JoinRules.PUBLIC
                             )
 
                 # Send the invite, if necessary.
@@ -570,7 +587,7 @@ class RegistrationHandler:
                     await room_member_handler.update_membership(
                         requester=create_requester(
                             self.hs.config.registration.auto_join_user_id,
-                            authenticated_entity=self._server_name,
+                            authenticated_entity=self.server_name,
                         ),
                         target=UserID.from_string(user_id),
                         room_id=room_id,
@@ -582,7 +599,7 @@ class RegistrationHandler:
                 # Send the join.
                 await room_member_handler.update_membership(
                     requester=create_requester(
-                        user_id, authenticated_entity=self._server_name
+                        user_id, authenticated_entity=self.server_name
                     ),
                     target=UserID.from_string(user_id),
                     room_id=room_id,
@@ -738,37 +755,20 @@ class RegistrationHandler:
             shadow_banned: Whether to shadow-ban the user
             approved: Whether to mark the user as approved by an administrator
         """
-        if self.hs.config.worker.worker_app:
-            await self._register_client(
-                user_id=user_id,
-                password_hash=password_hash,
-                was_guest=was_guest,
-                make_guest=make_guest,
-                appservice_id=appservice_id,
-                create_profile_with_displayname=create_profile_with_displayname,
-                admin=admin,
-                user_type=user_type,
-                address=address,
-                shadow_banned=shadow_banned,
-                approved=approved,
-            )
-        else:
-            await self.store.register_user(
-                user_id=user_id,
-                password_hash=password_hash,
-                was_guest=was_guest,
-                make_guest=make_guest,
-                appservice_id=appservice_id,
-                create_profile_with_displayname=create_profile_with_displayname,
-                admin=admin,
-                user_type=user_type,
-                shadow_banned=shadow_banned,
-                approved=approved,
-            )
+        await self.store.register_user(
+            user_id=user_id,
+            password_hash=password_hash,
+            was_guest=was_guest,
+            make_guest=make_guest,
+            appservice_id=appservice_id,
+            create_profile_with_displayname=create_profile_with_displayname,
+            admin=admin,
+            user_type=user_type,
+            shadow_banned=shadow_banned,
+            approved=approved,
+        )
 
-            # Only call the account validity module(s) on the main process, to avoid
-            # repeating e.g. database writes on all of the workers.
-            await self._account_validity_handler.on_user_registration(user_id)
+        await self._account_validity_handler.on_user_registration(user_id)
 
     async def register_device(
         self,
@@ -810,6 +810,7 @@ class RegistrationHandler:
         login_counter.labels(
             guest=is_guest,
             auth_provider=(auth_provider_id or ""),
+            **{SERVER_NAME_LABEL: self.server_name},
         ).inc()
 
         return (
@@ -859,9 +860,6 @@ class RegistrationHandler:
         refresh_token = None
         refresh_token_id = None
 
-        # This can only run on the main process.
-        assert isinstance(self.device_handler, DeviceHandler)
-
         registered_device_id = await self.device_handler.check_device_registered(
             user_id,
             device_id,
